@@ -14,6 +14,7 @@ package require Tcl 8.5
 package require asn 0.8.4
 
 ## Further dependencies
+package require aes
 package require des
 package require math::bignum
 package require md5 2
@@ -246,7 +247,6 @@ proc ::pki::_powm {x y m} {
 ### a^-1 === a^(m-2)    (all mod m)
 proc ::pki::_modi {a m} {
 	return [_powm $a [expr {$m - 2}] $m]
-	
 }
 
 proc ::pki::_oid_number_to_name {oid} {
@@ -733,6 +733,10 @@ proc ::pki::key {keylist {password ""} {encodePem 1}} {
 	if {$encodePem} {
 		set retval [::pki::_encode_pem $retval_parts(data) $retval_parts(begin) $retval_parts(end) $password]
 	} else {
+		if {$password != ""} {
+			return -code error "DER encoded keys may not be password protected"
+		}
+
 		set retval $retval_parts(data)
 	}
 
@@ -794,14 +798,54 @@ proc ::pki::_getopensslkey {password salt bytes} {
 	return $ret
 }
 
-proc ::pki::_encode_pem {data begin end {password ""}} {
+proc ::pki::_encode_pem {data begin end {password ""} {algo "aes-256-cbc"}} {
 	set ret ""
 
+	append ret "${begin}\n"
 	if {$password != ""} {
-		return -code error "Passwords are currently unsupported !"
+		switch -glob -- $algo {
+			"aes-*" {
+				set algostr [string toupper $algo]
+				set work [split $algo "-"]
+				set algo "aes"
+				set keysize [lindex $work 1]
+				set mode [lindex $work 2]
+				set blocksize 16
+				set ivsize [expr {$blocksize * 8}]
+			}
+			default {
+				return -code error "Only AES is currently supported"
+			}
+		}
+
+		set keybytesize [expr {$keysize / 8}]
+		set ivbytesize [expr {$ivsize / 8}]
+
+		set iv ""
+		while {[string length $iv] < $ivbytesize} {
+			append iv [::pki::_random -binary]
+		}
+		set iv [string range $iv 0 [expr {$ivbytesize - 1}]]
+
+		set password_key [::pki::_getopensslkey $password $iv $keybytesize]
+
+		set pad [expr {$blocksize - ([string length $data] % $blocksize)}]
+		append data [string repeat "\x09" $pad]
+
+		switch -- $algo {
+			"aes" {
+				set data [aes::aes -dir encrypt -mode $mode -iv $iv -key $password_key -- $data]
+			}
+		}
+
+		binary scan $iv H* iv
+		set iv [string toupper $iv]
+
+		append ret "Proc-Type: 4,ENCRYPTED\n"
+		append ret "DEK-Info: $algostr,$iv\n"
+		append ret "\n"
 	}
 
-	append ret "${begin}\n"
 	if {$::pki::rsa::base64_binary} {
 		append ret [binary encode base64 -maxlen 64 $data]
 	} else {
@@ -880,7 +924,7 @@ proc ::pki::_parse_pem {pem begin end {password ""}} {
 					set iv [binary format H* $iv]
 					set password_key [::pki::_getopensslkey $password $iv 24]
 
-					set pem [DES::des -dir decrypt -mode $mode -iv $iv -key $password_key $pem]
+					set pem [DES::des -dir decrypt -mode $mode -iv $iv -key $password_key -- $pem]
 				}
 				"AES-*" {
 					package require aes
@@ -900,7 +944,7 @@ proc ::pki::_parse_pem {pem begin end {password ""}} {
 					set iv [binary format H* $iv]
 					set password_key [::pki::_getopensslkey $password $iv [expr $keysize / 8]]
 
-					set pem [aes::aes -dir decrypt -mode $mode -iv $iv -key $password_key $pem]
+					set pem [aes::aes -dir decrypt -mode $mode -iv $iv -key $password_key -- $pem]
 				}
 			}
 		}
@@ -1584,7 +1628,13 @@ proc ::pki::_bits {num} {
 	return $ret
 }
 
-proc ::pki::_random {} {
+proc ::pki::_random args {
+	if {[lindex $args 0] == "-binary"} {
+		set outputmode binary
+	} else {
+		set outputmode numeric
+	}
+
 	if {![info exists ::pki::_random_dev]} {
 		foreach trydev [list /dev/urandom /dev/random __RAND__] {
 			if {$trydev != "__RAND__"} {
@@ -1619,6 +1669,15 @@ proc ::pki::_random {} {
 		}
 	}
 
+	switch -- $outputmode {
+		"numeric" {
+			# Do nothing, results are already numeric
+		}
+		"binary" {
+			set ret [binary format H* [format %02llx $ret]]
+		}
+	}
+
 	return $ret
 }
 
@@ -1822,4 +1881,4 @@ proc ::pki::rsa::generate {bitlength {exponent 0x10001}} {
 # # ## ### ##### ######## #############
 ## Ready
 
-package provide pki 0.4
+package provide pki 0.5
